"use client";

import { fetchLesson } from "@/actions/video";
import { LessonDescription, Questions } from "@/models/courseDetails";
import { equalsCheck } from "@/utils/helper";
import { useEffect, useState } from "react";
import { useSession } from "next-auth/react";
import axios from "axios";
import Link from "next/link";
import LoadingSpinner from "@/components/Layout/LoadingSpinner";
import QuizTimer from "@/components/Course/QuizTimer";
import timeConverter from "@/components/Course/timeConverter";
import { redirect } from "next/navigation";
import { useRouter } from "next/navigation";
import clientAxios from "@/services/config";
import { QUESTION_MARKS_API } from "@/services/api-end-point/questionMarks";

interface CourseQuizPageProps {
  params: {
    courseSlug: string;
    slug: string;
  };
}

interface OptionsType {
  answer: string;
  isAnswer: boolean;
  checked: boolean;
}

interface ExamSummeryType {
  mark?: number;
  totalMark?: number;
  totalCorrect?: number;

  totalWrong?: number;
}

export interface SessionProps {
  data?: {
    user?: {
      name?: string;
      email?: string;
      image?: string;
      id?: string;
    };
  };
}

const CourseQuiz = ({ params }: CourseQuizPageProps) => {
  const [lesson, setLesson] = useState<LessonDescription>({});
  const [question, setQuestion] = useState<Questions[]>([]);
  const [show, setShow] = useState<boolean>(false);
  const [errors, setErrors] = useState<any>();
  const [originalQuestion, setOriginalQuestion] = useState<any>({});
  const [examSummery, setExamSummery] = useState<ExamSummeryType>({});
  const [loading, setLoading] = useState<boolean>(false);
  const session = useSession() as SessionProps;
  const [timeTaken, setTimeTaken] = useState(null);

  const timeTakenResults: any = timeTaken && timeConverter(timeTaken);
  useEffect(() => {
    setLoading(true);
    fetchLesson(params.courseSlug).then((res) => {
      console.log(res);

      const result = res.questions?.map((item: Questions) => {
        const options =
          typeof item.answerOptions === "string" &&
          JSON.parse(item.answerOptions);
        const temp = {
          ...item,
          answerOptions: options.map((i: any) => ({
            ...i,
            checked: false,
          })),
        };
        return temp;
      });

      setLesson(res);
      // getTime(res);
      setOriginalQuestion(res);
      //   setQuestion(res.questions);
      setQuestion(result);
      setLoading(false);
    });
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const calculatedMarks = (
    markPerQuestion: number,
    negitiveMarks?: number | undefined
  ) => {
    let mark = 0;
    let totalCorrect = 0;
    let totalWrong = 0;
    let totalMark = 0;

    question?.forEach((question: Questions) => {
      let correctIndexes: any = [];
      let checkedIndexes: any = [];

      question?.answerOptions?.forEach((option: OptionsType, optionIndex) => {
        if (option?.isAnswer) correctIndexes.push(optionIndex);
        if (option?.checked) checkedIndexes.push(optionIndex);
      });
      if (equalsCheck(correctIndexes, checkedIndexes)) {
        mark += markPerQuestion || 1;
        totalCorrect++;
      } else {
        mark -= negitiveMarks || 0.25;
        totalWrong++;
      }
      totalMark += markPerQuestion || 1;
    });

    return { totalCorrect, totalWrong, totalMark, mark };
  };

  const handleCheck = (questionIndex: any, optionIndex: any, value: any) => {
    const questionTemp = [...question];
    questionTemp[questionIndex].answerOptions[optionIndex].checked = value;
    setQuestion(questionTemp);
  };

  const handleSubmit = async () => {
    const { mark, totalCorrect, totalMark, totalWrong } = calculatedMarks(
      lesson?.markPerQuestion,
      lesson?.negativeMarks
    );
    setExamSummery({
      mark,
      totalCorrect,
      totalMark,
      totalWrong,
    });
    const payload = {
      totalQuestion: question.length,
      mark,
      totalMark,
      negativeMark:
        totalWrong * (lesson?.negativeMarks ? lesson?.negativeMarks : 0),
      totalWrong,
      totalCorrect,
      questionType: "QUIZ",
      studentId: session?.data?.user ? session.data?.user?.id : "",
      studentName: session?.data?.user ? session.data?.user?.name : "",
      lessonId: lesson.id,
      lessonName: lesson.name,
      courseId: params.slug,
      isPassed: lesson?.passingMarks ? mark >= lesson?.passingMarks : true,
      //   assignmentId: "60a7b432d4b8cd0017febb23",
    };
    try {
      await clientAxios.post(
        `${QUESTION_MARKS_API.submit_quiz_by_student}`,
        payload
      );
      handleShow();
      window.alert(
        `time taken ${timeTakenResults?.hours}:${timeTakenResults?.minutes}: ${timeTakenResults?.seconds} `
      );
    } catch (error) {
      if (error instanceof Error) {
        setErrors(error.message);
      } else {
        setErrors("Something went wrong");
      }
    }
  };

  const handleShow = () => {
    setShow(true);
    setErrors(null);
    const result = originalQuestion?.questions?.map((item: Questions) => {
      const options =
        typeof item.answerOptions === "string" &&
        JSON.parse(item.answerOptions);
      const temp = {
        ...item,
        answerOptions: options.map((i: any) => ({
          ...i,
          checked: i.isAnswer,
        })),
      };
      return temp;
    });
    setQuestion(result);
  };

  // Disable right-click
  // useEffect(() => {
  //   const disableRightClick = (event: MouseEvent) => {
  //     event.preventDefault();
  //   };
  //   document.addEventListener("contextmenu", disableRightClick);
  //   return () => {
  //     document.removeEventListener("contextmenu", disableRightClick);
  //   };
  // }, []);

  // Disable copy and paste
  useEffect(() => {
    const disableCopyPaste = (event: ClipboardEvent) => {
      event.preventDefault();
    };
    document.addEventListener("copy", disableCopyPaste);
    document.addEventListener("cut", disableCopyPaste);
    document.addEventListener("paste", disableCopyPaste);

    return () => {
      document.removeEventListener("copy", disableCopyPaste);
      document.removeEventListener("cut", disableCopyPaste);
      document.removeEventListener("paste", disableCopyPaste);
    };
  }, []);

  // useEffect(() => {
  //   const handleVisibilityChange = () => {
  //     alert("Don't change the tab.");
  //   };
  //   document.addEventListener("visibilitychange", handleVisibilityChange);

  //   return () => {
  //     document.removeEventListener("visibilitychange", handleVisibilityChange);
  //   };
  // }, []);

  // Disable keyboard shortcuts
  useEffect(() => {
    const disableKeyboardShortcuts = (event: KeyboardEvent) => {
      if (event.ctrlKey || event.altKey || event.metaKey) {
        event.preventDefault();
      }
    };
    document.addEventListener("keydown", disableKeyboardShortcuts);
    return () => {
      document.removeEventListener("keydown", disableKeyboardShortcuts);
    };
  }, []);

  useEffect(() => {
    if (!session?.data?.user?.id) {
      redirect("/login");
    }
  }, [session]);

  if (loading) {
    return <LoadingSpinner isInitialLoading />;
  }

  if (question.length > 0) {
    return (
      <div className="container mx-auto px-4 sm:px-8 mt-4">
        <div className="mb-8">
          <h1 className="text-2xl font-bold text-gray-700 dark:text-white">
            {`Quizzes for "${question[0]?.lessonName}"`}
          </h1>
          {lesson?.markPerQuestion && (
            <p className="text-sm text-gray-700 dark:text-white">
              {`Each question ${lesson?.markPerQuestion} Mark`}
            </p>
          )}
        </div>
        {lesson?.examTime && (
          <div className="block sm:flex items-center sm:gap-3 gap-2">
            <div className="dark:text-gray-300 text-center sm:text-left text-black sm:text-base text-sm font-archivo font-normal m-2">
              Ending In:
            </div>
            <QuizTimer
              countdownTime={lesson?.examTime}
              setTimeTaken={setTimeTaken}
              handleSubmit={handleSubmit}
            />
          </div>
        )}

        {show && (
          <div className="rounded shadow-lg">
            <h1 className="text-xl font-bold text-gray-700 dark:text-white px-6 py-3">
              Quiz Summary
            </h1>
            <div className="relative overflow-x-auto w-full shadow-md sm:rounded-lg">
              <table className="w-full text-sm text-left rtl:text-right text-gray-500 dark:text-gray-400">
                <thead className="text-xs text-gray-700 uppercase dark:text-gray-400">
                  <tr>
                    <th
                      scope="col"
                      className="px-6 py-3 bg-gray-50 dark:bg-gray-800"
                    >
                      Total Marks
                    </th>
                    <th scope="col" className="px-6 py-3">
                      Total Correct Answer
                    </th>
                    <th
                      scope="col"
                      className="px-6 py-3 bg-gray-50 dark:bg-gray-800"
                    >
                      Total Wrong Answer
                    </th>
                    <th
                      scope="col"
                      className="px-6 py-3 bg-gray-50 dark:bg-gray-800"
                    >
                      Total Obtain Marks
                    </th>
                    <th scope="col" className="px-6 py-3">
                      Status
                    </th>
                  </tr>
                </thead>
                <tbody>
                  <tr className="border-b border-gray-200 dark:border-gray-700">
                    <th
                      scope="row"
                      className="px-6 py-4 font-medium text-gray-900 whitespace-nowrap bg-gray-50 dark:text-white dark:bg-gray-800"
                    >
                      {examSummery?.totalMark}
                    </th>
                    <td className="px-6 py-4">{examSummery?.totalCorrect}</td>
                    <td className="px-6 py-4 bg-gray-50 dark:bg-gray-800">
                      {examSummery?.totalWrong}
                    </td>
                    <td className="px-6 py-4 bg-gray-50 dark:bg-gray-800">
                      {examSummery?.mark}
                    </td>
                    <td className="px-6 py-4">
                      {lesson?.passingMarks && examSummery?.mark
                        ? examSummery?.mark! >= lesson?.passingMarks
                          ? "Pass"
                          : "Fail"
                        : "Pass"}
                    </td>
                  </tr>
                </tbody>
              </table>
            </div>
          </div>
        )}

        <div className="text-2 mt-10 flex items-center justify-end gap-1 text-xs">
          <span>
            একটি প্রশ্নের একাধিক উত্তর হতে পারে &amp;{" "}
            {lesson.isNegativeMarks
              ? `নেগেটিভ মার্কিং ${lesson.negativeMarks}`
              : "ভুল সিলেকশনে কোন নেগেটিভ মার্কিং নেই"}
          </span>
        </div>
        <div className="space-y-8 mt-2">
          {question?.map((quiz: Questions, i) => {
            const { _id, questionName, answerOptions, correctAnswer } = quiz;

            return (
              <div key={_id} className="quiz overflow-x-auto">
                <h4 className="question !text-gray-700 dark:!text-white text-md sm:text-3xl">{`Quiz ${
                  i + 1
                } - ${questionName}`}</h4>
                <form className={show ? "show" : "quizOptions"}>
                  {(answerOptions as OptionsType[])?.map(
                    (item: OptionsType, index: number) => {
                      const { answer, isAnswer, checked } = item;

                      return (
                        <label key={index} htmlFor={`option1_q_${i}_${index}`}>
                          <input
                            type="checkbox"
                            id={`option1_q_${i}_${index}`}
                            checked={checked}
                            onChange={(e) =>
                              handleCheck(i, index, e.target.checked)
                            }
                          />
                          {answer}
                        </label>
                      );
                    }
                  )}
                </form>
                {show && (
                  <div className="rounded p-2 bg-green-200 border border-red-400 w-full md:w-[48%] mt-4">
                    {correctAnswer}
                  </div>
                )}
              </div>
            );
          })}
        </div>
        {!show ? (
          <button
            type="button"
            className="px-7 py-2 w-32 rounded-full text-white bg-[#1971C2] block ml-auto mt-8 hover:opacity-90 active:opacity-100 active:scale-95 mb-4"
            onClick={handleSubmit}
          >
            Submit
          </button>
        ) : (
          <Link
            href={`/${params.slug}/course/${params.courseSlug}`}
            className="px-7 py-2 w-44  rounded-full text-white bg-[#1971C2] block ml-auto mt-8 hover:opacity-90 active:opacity-100 active:scale-95 mb-4"
          >
            Back to Lesson
          </Link>
        )}

        {errors && (
          <div className=" rounded p-2 bg-red-200 border border-red-400">
            {errors}
          </div>
        )}
      </div>
    );
  }
};

export default CourseQuiz;
